package cn.iocoder.yudao.module.system.controller.admin.commonlyused;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.system.controller.admin.commonlyused.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.commonlyused.CommonlyUsedDO;
import cn.iocoder.yudao.module.system.service.commonlyused.CommonlyUsedService;

@Tag(name = "管理后台 - 用户常用应用配置")
@RestController
@RequestMapping("/system/commonly-used")
@Validated
public class CommonlyUsedController {

    @Resource
    private CommonlyUsedService commonlyUsedService;

    @PostMapping("/create")
    @Operation(summary = "创建用户常用应用配置")
    @PreAuthorize("@ss.hasPermission('system:commonly-used:create')")
    public CommonResult<Long> createCommonlyUsed(@Valid @RequestBody CommonlyUsedSaveReqVO createReqVO) {
        return success(commonlyUsedService.createCommonlyUsed(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新用户常用应用配置")
    @PreAuthorize("@ss.hasPermission('system:commonly-used:update')")
    public CommonResult<Boolean> updateCommonlyUsed(@Valid @RequestBody CommonlyUsedSaveReqVO updateReqVO) {
        commonlyUsedService.updateCommonlyUsed(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除用户常用应用配置")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('system:commonly-used:delete')")
    public CommonResult<Boolean> deleteCommonlyUsed(@RequestParam("id") Long id) {
        commonlyUsedService.deleteCommonlyUsed(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得用户常用应用配置")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:commonly-used:query')")
    public CommonResult<CommonlyUsedRespVO> getCommonlyUsed(@RequestParam("id") Long id) {
        CommonlyUsedDO commonlyUsed = commonlyUsedService.getCommonlyUsed(id);
        return success(BeanUtils.toBean(commonlyUsed, CommonlyUsedRespVO.class));
    }

    @GetMapping("/list")
    @Operation(summary = "获得用户常用应用配置列表")
    @PreAuthorize("@ss.hasPermission('system:commonly-used:query')")
    public CommonResult<CommonlyUsedVO> getCommonlyUsedList() {
        CommonlyUsedVO vo = commonlyUsedService.getCommonlyUsedList();
        return success(vo);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出用户常用应用配置 Excel")
    @PreAuthorize("@ss.hasPermission('system:commonly-used:export')")
    @OperateLog(type = EXPORT)
    public void exportCommonlyUsedExcel(@Valid CommonlyUsedPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<CommonlyUsedDO> list = commonlyUsedService.getCommonlyUsedPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "用户常用应用配置.xls", "数据", CommonlyUsedRespVO.class,
                        BeanUtils.toBean(list, CommonlyUsedRespVO.class));
    }

}
