package cn.iocoder.yudao.module.system.service.commonlyused;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.system.controller.admin.auth.vo.AuthPermissionInfoRespVO;
import cn.iocoder.yudao.module.system.controller.admin.commonlyused.vo.CommonlyUsedPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.commonlyused.vo.CommonlyUsedRespVO;
import cn.iocoder.yudao.module.system.controller.admin.commonlyused.vo.CommonlyUsedSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.commonlyused.vo.CommonlyUsedVO;
import cn.iocoder.yudao.module.system.convert.auth.AuthConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.commonlyused.CommonlyUsedDO;
import cn.iocoder.yudao.module.system.dal.dataobject.permission.MenuDO;
import cn.iocoder.yudao.module.system.dal.mysql.commonlyused.CommonlyUsedMapper;
import cn.iocoder.yudao.module.system.service.permission.MenuService;
import cn.iocoder.yudao.module.system.service.permission.PermissionService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * 用户常用应用配置 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class CommonlyUsedServiceImpl implements CommonlyUsedService {

    @Resource
    private CommonlyUsedMapper commonlyUsedMapper;
    @Resource
    private PermissionService permissionService;
    @Resource
    private MenuService menuService;

    @Override
    public Long createCommonlyUsed(CommonlyUsedSaveReqVO createReqVO) {
        // 插入
        CommonlyUsedDO commonlyUsed = BeanUtils.toBean(createReqVO, CommonlyUsedDO.class);
        commonlyUsedMapper.insert(commonlyUsed);
        // 返回
        return commonlyUsed.getId();
    }

    @Override
    public void updateCommonlyUsed(CommonlyUsedSaveReqVO updateReqVO) {
        // 更新
        CommonlyUsedDO updateObj = BeanUtils.toBean(updateReqVO, CommonlyUsedDO.class);
        commonlyUsedMapper.updateById(updateObj);
    }

    @Override
    public void deleteCommonlyUsed(Long id) {
        // 删除
        commonlyUsedMapper.deleteById(id);
    }


    @Override
    public CommonlyUsedDO getCommonlyUsed(Long id) {
        return commonlyUsedMapper.selectById(id);
    }

    @Override
    public PageResult<CommonlyUsedDO> getCommonlyUsedPage(CommonlyUsedPageReqVO pageReqVO) {

        return commonlyUsedMapper.selectPage(pageReqVO);
    }

    @Override
    public CommonlyUsedVO getCommonlyUsedList() {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        CommonlyUsedVO vo = new CommonlyUsedVO();
        // -- 搜索我的常用
        List<CommonlyUsedDO> list = commonlyUsedMapper.selectList(new LambdaQueryWrapper<CommonlyUsedDO>()
                .eq(CommonlyUsedDO::getDeleted, 0).eq(CommonlyUsedDO::getUserId, userId).orderByAsc(CommonlyUsedDO::getSort));
        Map<Long, CommonlyUsedDO> map = list.stream().collect(Collectors.toMap(CommonlyUsedDO::getId, a -> a));
        List<CommonlyUsedRespVO> commonlyUsedRespVOS = new ArrayList<>();
        // -- 获取菜单列表
        List<MenuDO> menuList = new ArrayList<>();
        Set<Long> roleIds = permissionService.getUserRoleIdListByUserId(getLoginUserId());
        if (CollectionUtil.isNotEmpty(roleIds)) {
            Set<Long> menuIds = permissionService.getRoleMenuListByRoleId(roleIds);
            if (CollectionUtil.isNotEmpty(menuIds)) {
                menuList = menuService.getMenuList(menuIds);
                menuList.removeIf(menu -> !CommonStatusEnum.ENABLE.getStatus().equals(menu.getStatus()));
            }
        }
        List<String> menuIds = new ArrayList<>();
        List<String> processIds = new ArrayList<>();
        if (!list.isEmpty()) {
            // 区分出菜单和流程
            List<CommonlyUsedDO> myProcessList = list.stream().filter(a -> a.getType() == 1).collect(Collectors.toList());
            List<CommonlyUsedDO> myMenuList = list.stream().filter(a -> a.getType() == 2).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(myProcessList)) {
                processIds = myProcessList.stream().map(CommonlyUsedDO::getObjectId).collect(Collectors.toList());
            }
            if (CollectionUtil.isNotEmpty(myMenuList)) {
                menuIds = myMenuList.stream().map(CommonlyUsedDO::getObjectId).collect(Collectors.toList());
            }
            // -- 将菜单列表转树形
        }
        for (MenuDO menuDO : menuList) {
            menuDO.setSelectedFlag(false);
            if (menuIds.contains(String.valueOf(menuDO.getId()))) {
                menuDO.setSelectedFlag(true);
                CommonlyUsedRespVO commonlyUsedRespVO = BeanUtils.toBean(menuDO, CommonlyUsedRespVO.class);
                commonlyUsedRespVO.setImg(menuDO.getIcon());
                commonlyUsedRespVO.setName(menuDO.getName());
                commonlyUsedRespVOS.add(commonlyUsedRespVO);
            }
        }
        // -- 获取流程列表


        List<AuthPermissionInfoRespVO.MenuVO> menus = AuthConvert.INSTANCE.buildMenuTree(menuList);
        vo.setMenus(menus);
        vo.setCommonlyUsedRespVOS(commonlyUsedRespVOS);

        // TODO: 2024/9/5 还差流程的实现
        return null;
    }

}
